# HPROC1A - Run CCDPROC on a Hectospec or Hectochelle image
# July 10, 2008
# By Doug Mink, SAO Telescope Data Center

procedure hproc1a (image)

string	image=""	{prompt="Images to process"}

string	biassec="[1080:1120,3:4610]"	{prompt="Bias section"}
bool	fixpix=yes	{prompt="Fix bad pixels (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp2, imin, imu
string	imx, imsec, history
bool	vb, sv

	vb = verbose
	sv = saveold
	imx = ".fits"
	imin = image

	if (imin == "") {
	    print ("HPROC1: Run CCDPROC on a Hectospec/Hectochelle image")
	    print ("Usage: hproc1 <input image with 4 extensions>")
	    return
	    }

# Make directory in which to save preampfixed multi-extension files
	if (!access ("Unproc"))
	    mkdir ("Unproc")

# Do not process images without extensions
	imgetp.default = "-1"
	imsec = imin // "[0]"
	imgetp (imsec, "i_naxis")
	if (imgetp.value != "0") {
	    print ("HPROC1: ", imin, " is not a multi-extension image")
	    print ("NAXIS = ", imgetp.value)
	    return
	    }

# Do not process image twice
	imgetp.default = "0"
	imsec = imin // "[1]"
	imgetp (imsec, "HPROC")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HPROC1: ", imin, " has already been processed")
		print ("HPROC = ", imgetp.value)
		}
	    return
	    }

# Find root name of image
	imroot = imin
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imageout
	    imin = imin // imx
	    }
#	if (vb) {
#	    print ("HPROC: image root is ",imroot)
#	    }

#  Run CCDPROC only if it has not already been run
	imgetp.default = "0"
	imsec = imin // "[1]"
	imgetp (imsec, "CCDPROC")
	if (imgetp.value != "0") {
	    print ("HPROC1: CCDPROC has already been run on", imin)
	    print ("CCDPROC: ", imgetp.value)
	    return
	    }

#  Trim, subtract overscan, and fix bad pixels using mscred.ccdproc
	ccdproc.bpmasks =     ""
	ccdproc.ccdtype =     ""
	ccdproc.noproc =      no
	ccdproc.xtalkcor =    no
	ccdproc.fixpix =      fixpix
	ccdproc.overscan =    yes
	ccdproc.trim =        yes
	ccdproc.zerocor =     no   # leave up to specroad interaction
	ccdproc.darkcor =     no   # leave up to specroad interaction
	ccdproc.flatcor =     no
	ccdproc.sflatcor =    no
	ccdproc.split =       no
	ccdproc.merge =       no
	ccdproc.xtalkfile =   ""
	ccdproc.fixfile =     "BPM"
	ccdproc.saturation =  "INDEF"
	ccdproc.sgrow =       0
	ccdproc.bleed =       "INDEF"
	ccdproc.btrail =      20
	ccdproc.bgrow =       0
	ccdproc.biassec =     biassec
	ccdproc.trimsec =     "!TRIMSEC"
	ccdproc.zero =        "ZERO"  # leave up to specroad interaction
	ccdproc.dark =        "DARK"  # leave up to specroad interaction
	ccdproc.flat =        "FLAT"
	ccdproc.sflat =       ""
	ccdproc.minreplace =  1.
	ccdproc.interactive = no
	ccdproc.function =    "legendre"
	ccdproc.order =       1
	ccdproc.sample =      "*"
	ccdproc.naverage =    1
	ccdproc.niterate =    1
	ccdproc.low_reject =  3.
	ccdproc.high_reject = 3.
	ccdproc.grow =        0.
	ccdproc.fd =          ""
	ccdproc.fd2 =         ""
	temp2 = mktemp (imroot) // "pc" // imx
	if (verbose)
	    print ("HPROC1: fixpix, overscan, and trim ", imin, " -> ", temp2)
	mscred.ccdproc (imin, output=temp2)

#  Rename original file
	if (sv) {
	    imu = "Unproc/" // imin
	    if (access (imu))
		delete (imu)
	    rename (imin, imu)
	    }
	else {
	    delete (imin)
	    }

#  Rename result to same as original file
	rename (temp2, imin)
	if (verbose)
	    print ("HPROC: ",temp2," -> ", imin)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " fixpix, overscan, and trim"
	imsec = imin // "[1]"
        hedit (imsec, "HPROC", history, add+, verify-)
	imsec = imin // "[2]"
        hedit (imsec, "HPROC", history, add+, verify-)
	imsec = imin // "[3]"
        hedit (imsec, "HPROC", history, add+, verify-)
	imsec = imin // "[4]"
        hedit (imsec, "HPROC", history, add+, verify-)

end

# Feb 22 2008	New program based on old HPROC
# Jul 10 2008	Uncomment zero and dark parameters for HSPROC use
