# HSHIFT - Compare image to reference and put shifts in header
# May 11, 2005
# By Doug Mink, SAO Telescope Data Center

procedure hshift (images)

string	images=""	{prompt="Image file(s) for which to compute shifts"}

file	refimage=""	{prompt="Reference night sky image file (close to flat in time)"}
file	imdir=""	{prompt="Directory containing images"}
string	regions=""	{prompt="Regions to cross-correlate"}
int	window=31	{prompt="Width of correlation window"}
string	extension=""	{prompt="Image extension if any"}
bool	raw=yes		{prompt="Is image raw (yes) or amplifiers aligned (no)"}
bool	saveshift=no	{prompt="Save shift values in image header (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print shifts (y or n)"}

begin

file	imroot, imin, templist, imrref
file	imfile, newfile, imex0, imref
string	imx, ims, history, tchar, reg, sdx, sdy, sdate, ext, extx
string	dateobs, dir
real	dx, dy, mjd, mjd0
bool	dellist, vb, save, extfound, dirfound
int	fchar, lchar, win

	vb = verbose
	imx = ".fits"
	ims = images
	imref = refimage
	reg = regions
	save = saveshift
	extx = extension
	win = window
	dir = imdir

	if (ims == "") {
	    print ("HSHIFT: Compare image to reference and shift it to match")
	    print ("Usage: hshift <input image>")
	    print ("Usage: hshift @<input image list file>")
	    return
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
#	else if (dir != "") {
#	    impaths = dir // "/" // ims
#	    templist = mktemp ("shiftlist")
#	    files (impaths, >templist)
#	    }
	else {
	    templist = mktemp ("shiftlist")
	    files (ims, >templist)
	    }
	if (vb) {
	    print ("HSHIFT: Reading from list file ", templist)
	    print ("HSHIFT: Reference image is ", imref)
	    }

#  Use earliest file in list as reference if none is set
	if (imref == "") {
	    mjd0 = 0.0
            list = templist
            while (fscan (list,imin) != EOF) {
		imgetp.default = "0"
		imgetp (imin, "MJD")
		if (imgetp.value != "0") {
		    mjd = real (imgetp.value)
		    }
		if (mjd0 == 0.0 || mjd < mjd0) {
		    mjd0 = mjd
		    imref = imin
		    }
		}
	    if (vb) {
		imgetp (imref, "DATE-OBS")
		dateobs = imgetp.value
		print ("HSHIFT: ", imref, " at ",dateobs," used as reference image")
		}
	    }

	if (extx != "") {
	    ext = "[" // extx // "]"
	    imref = imref // ext
	    }
	else {
	    ext = "[0]"
	    }

# Find root name of reference image
	imrref = imref
	smatch (imrref, imx)
	extfound = smatch.found
	if (extfound) {
	    lchar = smatch.start - 1
	    }
	else {
	    lchar = 100
	    }
	smatch (imrref, "/200")
	dirfound  = smatch.found
	if (extfound) {
	    if (dirfound) {
		fchar = smatch.start + 11
		}
	    else {
		fchar = 1
		}
	    imrref = substr (imref,fchar, lchar)
	    }
	else {
	    if (dirfound) {
		fchar = smatch.start + 11
		imrref = substr (imref,fchar, lchar)
		}
	    else {
		imrref = imref
		}
	    }

        list = templist
        while (fscan (list,imin) != EOF) {

	
# Do not shift image twice
	    imgetp.default = "0"
	    imex0 = imin // ext
	    imgetp (imex0, "HSHIFT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HSHIFT: ", imex0, " shift has already been computed")
		    print ("HSHIFT = ", imgetp.value)
		    }
		next
		}

# Find root name of image
	    imroot = imin
	    smatch (imroot, imx)
	    extfound = smatch.found
	    if (extfound) {
		lchar = smatch.start - 1
		}
	    else {
		lchar = 100
		}
	    smatch (imroot, "/200")
	    dirfound  = smatch.found
	    if (extfound) {
		if (dirfound) {
		    fchar = smatch.start + 11
		    }
		else {
		    fchar = 1
		    }
		imroot = substr (imroot,fchar, lchar)
		}
	    else {
		if (dirfound) {
		    fchar = smatch.start + 11
		    imroot = substr (imroot,fchar, lchar)
		    }
		else {
		    imroot = imin
		    }
		}
#	    if (vb) {
#		print ("HSHIFT: image root is ",imroot)
#		}

#  Find out whether there has been a significant shift in selected region
	    imex0 = imin // ext
	    x2register (imex0, imref, 0, 0, xwindow=win, ywindow=win, regions=reg, verbose=no)
	    dx = x2register.xcdx
	    if (raw && (extx == "2" || extx == "4")) {
		dx = -dx
		}
	    dy = x2register.xcdy
	    sdx = dx
	    sdy = dy
	    imgetp.default = "0"
	    imgetp (imex0, "DATE-OBS")
	    sdate = imgetp.value
	    if (vb) {
		printf ("%s %s %s %s %s %s %7.4f %7.4f\n", imroot, extx, imrref, extx, reg, sdate, dx, dy)
		}

# Make history line including time for shift correction
	    if (save) {
        	gettime ()
		history = gettime.date // " " // gettime.time // " " // reg // " dx=" // dx // " dy=" // dy
		hedit (imin, "HSHIFT", history, add+, verify-)
		hedit (imin, "HSHIFTX", sdx, add+, verify-)
		hedit (imin, "HSHIFTY", sdy, add+, verify-)
		}

	    }
	if (deltemp && tchar != '@') {
	    delete (templist)
	    }
end

# Nov 22 2004	Program to compute image shifts and write to the image header
# Dec 29 2004	Add optional image extension name or number

# Jan  5 2005	Drop directory from output line
# Jan  5 2005	Change sign of dx for reversed extensions 2 and 4
# Apr 19 2005	Fix prompt and add window parameter
# May  5 2005	If no reference image is given, use earliest in list of input files
$ May 11 2005	Write HSHIFTX and HSHIFTY to the header, if requested
