# HSKYFIT1 - Subtract rescaled sky from object+sky spectrum
# September 24, 2008
# By Doug Mink, SAO Telescope Data Center

procedure hskyfit1 (objfile, skyfile, outfile)

file	objfile=""	{prompt="Object+sky spectrum file"}
file	skyfile=""	{prompt="Sky spectrum file"}
file	outfile=""	{prompt="Output object,object+sky,sky multispec file"}

int	objap=1		{prompt="Aperture for object+sky spectrum"}
int	objband=1	{prompt="Band for object+sky spectrum"}
file	objdir=""	{prompt="Directory containing object+sky spectrum"}
real	fitmax=2.0	{prompt="Maximum obj+sky/sky for sky fit, else 1:1"}
int	skyap=1		{prompt="Aperture for sky spectrum"}
int	skyband=1	{prompt="Band for sky spectrum"}
file	skydir=""	{prompt="Directory containing sky spectrum"}
file	outdir=""	{prompt="Directory for output spectrum"}
bool    plotobj = no    {prompt="Plot spectrum before sky subtraction (y or n)"}
bool    plotres = no    {prompt="Plot sky-subtracted spectrum (y or n)"}
bool	verbose=no	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print debugging information (yes or no)"}

begin
	file ims, imin, fname, inspec, outspec
	bool vb

	inspec = objfile
	outspec = outfile
	vb = verbose

	if (!defpac ("svdfit")) {
	    svdfit
	    }

#  Set up parameters for fit of sky to object + sky
	svdres.spectra = inspec
	svdres.specnum = objap
	svdres.specband = objband
	svdres.specdir = objdir
	svdres.eigenfile = skyfile
	svdres.eigenum = skyap
	svdres.eigenband = skyband
	svdres.fitmax = fitmax
	svdres.resfile = outspec
	svdres.resdir = outdir
	svdres.res_format = "multispec"
	svdres.obj_plot = plotobj
	svdres.vec_plot = no
	svdres.res_plot = plotres
	svdres.plot_int = plotobj
	svdres.interp_mode = "spline3"
	svdres.nbin = INDEF
	svdres.fixbad = "fill"
	svdres.badlines = "badlines.dat"
	svdres.cont_remove = "subtract"
	svdres.vec_cont = "subtract"
	svdres.cont_add = yes
	svdres.st_lambda = INDEF
	svdres.end_lambda = INDEF
	svdres.verbose = vb
	svdres.debug = debug

#	dpar ("svdres",>"svdres.par")
	svdres (inspec,outspec)

	if (vb) {
	    print (inspec," background removed using SVDRES")
	    }
end

# Mar  6 2006	New task based on resub.cl
# Mar 17 2006	Change prompts from aperture to band
# Nov 16 2006	Add obj+sky/sky maximum, fitmax, for fit of sky to obj+sky
# Nov 20 2006	Add objap (obj+sky aperture) and skyap (sky aperture) params
# Nov 20 2006	Add plotres parameter to plot residuals

# Sep 24 2008	Add plotobj parameter to plot and possibly edit object spectrum
