# HSPLIT - Split a single multispec hectospec file into 1-D spectra
# By Doug Mink, SAO Telescope Data Center
# February 13, 2007

procedure hsplit (msfile)

string	msfile=""	{prompt="Aperture reference multispec file (drop .ms)"}
bool	deltemp=yes	{prompt="Delete temporary list file (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print more information (y or n)"}

begin

	file	keyfile, keypath, imin, imroot, imnew
	file	tempfile, tempfile2, outroot, temparg, newpath
	int	lroot, aperture, nfiles, i
	string	arg, tchar, ims, history, dir1d, aparg
	string	keyword, apkwr, apval1, type, x1, x2, range
	string	name, ra, dec, catnum, fiber, x, y
	string	apkey, fpkey, objkey, skykey, fibkey
	string	pra, pdec, pepoch, apname
	bool	vb, db

	vb = verbose
	db = debug
	imin = msfile
#	keyfile = osfn (keywords)
	keyfile = ""

#  Set defaults for scopy program
	scopy.w1 = INDEF
	scopy.w2 = INDEF
	scopy.bands = "1"
	scopy.beams = ""
	scopy.apmodulus = 0
	scopy.format = "onedspec"
	scopy.renumber = no
	scopy.offset = 0
	scopy.clobber = yes
	scopy.merge = yes
	scopy.rebin = no
	scopy.verbose = yes

	if (imin == "") {
	    print "hsplit: Make 1D spectra (fiber.object) from Hectospec multispec file"
	    return
	    }

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

# Make sure that input is a multispec file
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imin, 1, lroot)
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imnew = imin // ".fits"
		if (access (imnew)) {
		    imin = imnew
		    }
		else {
		    print ("HSPLIT: ", imin," is not a multispec file")
		    return
		    }
		}
	    }
        else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		imnew = imroot // ".ms.fits"
		if (access (imnew)) {
		    imin = imnew
		    }
		else {
		    print ("HSPLIT: ", imin," is not a multispec file")
		    return
		    }
		}
	    else {
		imroot = imin
		imin = imroot // ".ms.fits"
		if (!access (imin)) {
		    print ("HSPLIT: ", imin," is not a multispec file")
		    return
		    }
		}
            }

# COMMENT OUT this section if HSPLIT is expected to always run
# Do not split an image twice
#	imgetp.default = "0"
#	imgetp (imin, "HSPLIT")
#	if (imgetp.value != "0") {
#	    if (vb) {
#		print ("HSPLIT: ", imin, " has already been split")
#		print ("HSPLIT = ", imgetp.value)
#		}
#	    return
#	    }


	if (vb) {
	    print ("Delete keywords listed in file ",imin)
	    }
	if (vb) {
	    print ("Object name and position from ", imin)
	    }

# Make separate file to pre-edit header for output files
	tempfile = imroot // "t.ms.fits"
	if (access (tempfile)) {
	    delete (tempfile)
	    }

# Copy .fits or .imh output from DOFIBERS into file to be edited
	imcopy (imin, tempfile)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	hedit (tempfile, "HSPLIT", history, add+, verify-, show-)

# Get number of spectra in multispec file
	imgetp.default = "0"
	imgetp (imin, "i_naxis2")
	nfiles = int (imgetp.value)
	aparg = "1-" // nfiles

# If 1d directory does not exist, create it
	dir1d = "1d." // imroot
	if (!access (dir1d)) {
	    if (vb)
		print ("HSPLIT: Making directory ", dir1d)
	    mkdir (dir1d)
	    }
	outroot = dir1d // "/temp"

# Extract all spectra into a separate FITS file
	if (vb) {
	    print ("scopy ", tempfile, " ", outroot, " apertures=", aparg)
	    }
	scopy (tempfile, outroot, apertures=aparg)

	for (i = 1; i <= nfiles; i = i + 1) {

	    if (i < 10) {
		apname = "000" // i
		}
	    else if (i < 100) {
		apname = "00" // i
		}
	    else if (i < 1000) {
		apname = "0" // i
		}
	    else {
		apname = i
		}
	    tempfile2 = dir1d // "/temp." // apname // ".fits"

	    haphead (tempfile2, verbose=vb, debug=db)
	    name = haphead.newname

# Rename file to aperture number and object name
	    newpath = dir1d // "/" // name
	    rename (tempfile2, newpath)
	    if (vb) {
		print ("rename ",tempfile2, " ", newpath)
		}
	    }

	if (deltemp) {
	    delete (tempfile)
	    }

# Add history line including time to original multispec file
	gettime ()
	history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	if (vb) {
	    hedit (imin, "HSPLIT", history, add+, verify-, show+)
	    }
	else {
	    hedit (imin, "HSPLIT", history, add+, verify-, show-)
	    }
	return
end

# Feb 22 2000	New program based on split2df
# May 25 2000	Add code to deal with possible input FITS file

# Oct  9 2003	Add -k option to specify keyword deletion list file
# Oct 28 2003	Update parsing code and rewrite to work on actual data
# Oct 31 2003	Save name from APIDn keyword value; use for filename if not target

# Apr 15 2004	Write APNUM1 keyword to 1D files with APNUMi value

# Mar  7 2005	Turn special characters in objet names into _
# Mar  7 2005	Set dispaxis=1
# Apr 11 2005	Run using /data/oir/bin/ksh instead of /data/mmti/bin/ksh
# Sep  8 2005	Convert to CL script from KSH script
# Sep 16 2005	Send fiberkey to correct output file

# Jul 13 2006	Use onedspec.scopy instead of imextract to split .ms files
# Aug 17 2006	Replace hsplit.cl with this
# Aug 23 2006	Write pointing FRA and FDEC as field center to FITS headers
# Sep  7 2006	Really write pointing PRA and PDEC to FITS header
# Sep  7 2006	Compute fiber area and add to FITS header as FIBAREA
# Oct 10 2006	Rename multispec file APERTURE keyword POINTING
# Oct 10 2006	Split sethead command into two pieces
# Dec 13 2006	Write SETHEAD arguments to single-line file to save calls
# Dec 14 2006	Return area from HFIBAREA and set in header with SETHEAD
# Dec 14 2006	Read plate center from CAT-RA/CAT-DEC if RA/DEC not there
# Dec 19 2006	Write all new keyword values in HAPHEAD and return object name

# Feb 13 2007	Fix bug so that unsplit multispec file is preserved
