# XCSKY1 - Piecewise cross-correlate a spectrum against a template in pixel
#          space using MKXCTEMPW and XCSAO, make new cal file id file, and
#          reidentify
# November 4, 2003
# By Doug Mink, SAO Telescope Data Center

procedure xcsky1 (object, tempseg)

file	object=""	{prompt="Object spectrum file"}
file	tempseg=""	{prompt="File of template segments"}

int	objap=1		{prompt="Object aperture"}
int	objband=0	{prompt="Object band"}
string	caltemp=""	{prompt="Template calibration spectrum file"}
string	calshift=""	{prompt="Shifted and stretched calibration file"}
int	nbin=256	{prompt="Number of points to cross-correlate (rebinned)"}
bool	intxcor=no	{prompt="Run xcsao interactively (y or n)"}
real	ndev=0		{prompt="Reject past this number of standard deviations"}
char	xcmode="pixel"	{prompt="Correlate in pixel, wavelength, or velocity"}
real	rmin=2.0	{prompt="Reject segment cross-correlations with R < this"}
bool	intdisp=no	{prompt="Fit dispersion function interactively (y or n)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	char arg
	string templist	# File to which to write list of segments
	string xcout	# File to which to write xcsao output
	file objfile, tempfile, ctempfile, ttempfile, ttempfile1
	string dbfile	# id file created by reref
	string dbfile1	# reref output id file renamed to match correlated file
	int band, ap

	real constant
	real nd
	int im
	bool intact
	char xmode

	if (ndev < 1) {
	    nd = 10
	    }
	else {
	    nd = ndev
	    }
	intact = intdisp
	objfile = object
	ap = objap
	band = objband
	templist = tempseg
	ctempfile = caltemp
	ttempfile = calshift
	xmode = xcmode

# Cross-correlate spectrum against template, producing output in xcout
	rvsao (motd=no)
	arg = "@" // templist
#	arg = templist
	xcout = mktemp ("xcsao")
	if (debug) {
	    print ("xcsao(",objfile,",specnum=",ap,",templates=",arg,",tempband=",band,",correlate=",xmode,",ncols=",nbin)
	    }
	xcsao (objfile,specnum=ap,templates=arg,tempband=band,correlate=xmode,ncols=nbin,report_mode=16,logfile=xcout,curmode=intxcor,debug=debug)
	if (debug) {
	    print ("xcsao output written to ",xcout)
	    }

# If spectrum file does not exist for shifted template, create it
	if (!access (ttempfile)) {
	    ttempfile1 = ttempfile // ".fits"
	    if (!access (ttempfile1)) {
		printf ("Fake template file %s does not exist\n", ttempfile)
#		imcopy (ctempfile, ttempfile)
		}
	    }
	else if (debug) {
	    printf ("Fake template file is %s\n",ttempfile)
	    }

# Read pixels and shifts from xcout and fit a curve to them, then make
# a new database/id file with modified pixel positions for lines
	
	reref (objfile, ctempfile, ttempfile, rmin=rmin, smin=nd, shiftfile=xcout, verbose=verbose, debug=debug)

# Rename id file from cross-correlation
	dbfile = "database/id" // ttempfile
	dbfile1 = "database/id" // objfile
	if (debug) {
	    printf ("%s -> %s\n", dbfile, dbfile1)
	    }
	rename (dbfile, dbfile1)

	delete (xcout)
end

# Jul 21 2003	New task
